<?php

namespace App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * GeoCountries
 *
 * @ORM\Table(name="geo_countries", indexes={@ORM\Index(name="geo_countries_continent_id_foreign", columns={"continent_id"})})
 * @ORM\Entity
 */
class GeoCountries
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="name", type="string", length=255, nullable=false)
     */
    private $name;

    /**
     * @var string
     *
     * @ORM\Column(name="cca2", type="string", length=3, nullable=false)
     */
    private $cca2;

    /**
     * @var string
     *
     * @ORM\Column(name="cca3", type="string", length=3, nullable=false)
     */
    private $cca3;

    /**
     * @var int
     *
     * @ORM\Column(name="ccn3", type="integer", nullable=false)
     */
    private $ccn3;

    /**
     * @var int
     *
     * @ORM\Column(name="sub_continent_id", type="integer", nullable=false, options={"unsigned"=true})
     */
    private $subContinentId = '0';

    /**
     * @var \DateTime|null
     *
     * @ORM\Column(name="created_at", type="datetime", nullable=true)
     */
    private $createdAt;

    /**
     * @var \DateTime|null
     *
     * @ORM\Column(name="updated_at", type="datetime", nullable=true)
     */
    private $updatedAt;

    /**
     * @var \GeoContinents
     *
     * @ORM\ManyToOne(targetEntity="GeoContinents")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="continent_id", referencedColumnName="id")
     * })
     */
    private $continent;

    /**
     * @ORM\OneToMany(targetEntity=Station::class, mappedBy="pays", orphanRemoval=true)
     */
    private $stations;

    public function __construct()
    {
        $this->stations = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getCca2(): ?string
    {
        return $this->cca2;
    }

    public function setCca2(string $cca2): self
    {
        $this->cca2 = $cca2;

        return $this;
    }

    public function getCca3(): ?string
    {
        return $this->cca3;
    }

    public function setCca3(string $cca3): self
    {
        $this->cca3 = $cca3;

        return $this;
    }

    public function getCcn3(): ?int
    {
        return $this->ccn3;
    }

    public function setCcn3(int $ccn3): self
    {
        $this->ccn3 = $ccn3;

        return $this;
    }

    public function getSubContinentId(): ?int
    {
        return $this->subContinentId;
    }

    public function setSubContinentId(int $subContinentId): self
    {
        $this->subContinentId = $subContinentId;

        return $this;
    }

    public function getCreatedAt(): ?\DateTimeInterface
    {
        return $this->createdAt;
    }

    public function setCreatedAt(?\DateTimeInterface $createdAt): self
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getUpdatedAt(): ?\DateTimeInterface
    {
        return $this->updatedAt;
    }

    public function setUpdatedAt(?\DateTimeInterface $updatedAt): self
    {
        $this->updatedAt = $updatedAt;

        return $this;
    }

    public function getContinent(): ?GeoContinents
    {
        return $this->continent;
    }

    public function setContinent(?GeoContinents $continent): self
    {
        $this->continent = $continent;

        return $this;
    }
    
    public function __toString() {
        return $this->name;
    }

    /**
     * @return Collection|Station[]
     */
    public function getStations(): Collection
    {
        return $this->stations;
    }

    public function addStation(Station $station): self
    {
        if (!$this->stations->contains($station)) {
            $this->stations[] = $station;
            $station->setPays($this);
        }

        return $this;
    }

    public function removeStation(Station $station): self
    {
        if ($this->stations->contains($station)) {
            $this->stations->removeElement($station);
            // set the owning side to null (unless already changed)
            if ($station->getPays() === $this) {
                $station->setPays(null);
            }
        }

        return $this;
    }


}
